{ Unit otte_ctype contains system-dependant parts of OTTE. Namely, it provides
  OTTE witch basic charset routines, used for case-insensivity purposes.
  This example was written for linux and is strongly GNU libc dependant.
------------------------------------------------------------------------------}

{$I otte_cfg.pas}
unit otte_ctype;

interface //===================================================================

type
  TotteCharProperty = array[char] of boolean;
  TotteCharTransfToLower = array[char] of char;

function getlocale: ansistring;
function load_charmap(charmap: ansistring): boolean;

var
  isalpha: TotteCharProperty;
  tolower: TotteCharTransfToLower;

implementation //==============================================================

type
  wchar_t = longint;  //actually a glibc widechar
  iconv_t = pointer;

const
  LC_CTYPE = 0;

function c_setlocale(category: longint; locale: pchar): pchar;
cdecl; external 'c' name 'setlocale';

function c_isalpha(c: longint): longint;
cdecl; external 'c' name 'isalpha';

function c_iswalpha(c: wchar_t): longint;
cdecl; external 'c' name 'iswalpha';

function c_tolower(c: longint): longint;
cdecl; external 'c' name 'tolower';

function c_towlower(c: wchar_t): wchar_t;
cdecl; external 'c' name 'towlower';

function c_iconv_start(const tocode: PChar; const fromcode: PChar): iconv_t;
cdecl; external 'c' name 'iconv_open';

function c_iconv_close(cd: iconv_t): longint;
cdecl; external 'c' name 'iconv_close';

function c_iconv(cd: iconv_t;
  var inbuf:  pointer; var inbytesleft:  longint;
  var outbuf: pointer; var outbutesleft: longint): integer;
cdecl; external 'c' name 'iconv';

//-----------------------------------------------------------------------------
function getlocale: ansistring;
begin
  result := c_setlocale(LC_CTYPE, nil);
end;

//-----------------------------------------------------------------------------
function setlocale(const locale: ansistring): boolean;
begin
  result := c_setlocale(LC_CTYPE, pchar(locale)) <> nil;
end;

//-----------------------------------------------------------------------------
function load_charmap(charmap: ansistring): boolean;

var
  inbytesleft, outbytesleft: integer;
  inbuf, outbuf: pointer;
  wc: array[char] of wchar_t;
  cd: iconv_t;
  c: char;
  converted: TotteCharProperty;

begin
  result := true;

  //try to start the locale
  if not setlocale('') then begin
    result := false;
    exit;
  end;
  
  if charmap = '' then //Charmap undefined, so we rely on system locale
  for c := #0 to #255 do begin
    isalpha[c] := (c_isalpha(byte(c)) <> 0);
    tolower[c] := char(c_tolower(byte(c)));
  end
  else begin
    //Charmap may be different than in system locale, so we use iconv to convert
    //all to WCHAR_T and pray that c_iswalpha and c_towlower support them all.

    //start iconv
    cd := c_iconv_start('WCHAR_T', PChar(charmap));
    if longint(cd) = -1 then begin
      result := false;
      exit;
    end;

    //we convert all one-byte chars them to widechars
    try
      for c := #00 to #255 do begin
        inbuf := addr(c);
        inbytesleft := sizeof(c); //1
        outbuf := addr(wc[c]);
        outbytesleft := sizeof(wchar_t); //4

        //every one-byte char can be converted to WCHAR_T, unless it's unused.
        //the problem here is that multibyte encodings are not detected...
        if c_iconv(cd, inbuf, inbytesleft, outbuf, outbytesleft) = -1 then begin
          converted[c] := false;
          isalpha[c] := false;
          tolower[c] := c;
        end
        else converted[c] := true;
      end;
    except
      result := false;
    end;

    //try to close iconv
    if (c_iconv_close(cd) <> 0) or not result then begin
      result := false;
      exit;
    end;

    //fill isalpha
    for c := #00 to #255 do if converted[c] then
      isalpha[c] := (c_iswalpha(wc[c]) <> 0);

    //now convert widechars to lowercase
    for c := #00 to #255 do
      if converted[c] then
        if isalpha[c] then wc[c] := c_towlower(wc[c])
        else tolower[c] := c;

    //now we shall convert lowercase widechars back to one-byte chars.
    //if conversion for a char impossible, we use the single-byte char
    //from _before_ any conversions.

    //start iconv again
    cd := c_iconv_start(PChar(charmap), 'WCHAR_T');
    if longint(cd) = -1 then begin
      result := false;
      exit;
    end;

    //we convert all widechars to one-byte chars
    //try..except statement must be used
    try
      for c := #00 to #255 do if converted[c] and isalpha[c] then begin
        inbuf := addr(wc[c]);
        inbytesleft := sizeof(wchar_t); //4
        outbuf := addr(tolower[c]);
        outbytesleft := sizeof(tolower[c]); //1

        //not every widechar can be converted to a sibgle-byte char
        //in multibyte charmaps there may be strange things...
        if c_iconv(cd, inbuf, inbytesleft, outbuf, outbytesleft) = -1 then
          tolower[c] := c;
      end;
    except
      result := false;
    end;

    //try to close iconv
    if (c_iconv_close(cd) <> 0) or not result then begin
      result := false; //What a #$%#@@!!! The success was so close...
      exit;
    end;
  end;
end;

//-----------------------------------------------------------------------------
begin
  load_charmap('');
end.
