unit tengMessages;

{$I otte/otte_cfg.pas}

interface

const
  ProgramVersion   = 'Teng 0.9.5-1';
  ProgramAuthor    = 'Michal Nowakowski <miszka@limes.com.pl>';
  ProgramCopyright = 'Copyright (C) 2005 Free Software Foundation, Inc.';
  
  ProgramUsage     = 'teng -m mode [-i input] [-o output] [-l locale] [-Bxtrcdbq] [-hu]';

  msgPolicyBegin     = 0;
  msgPolicyEnd       = 3;
  msgUsage           = 4;
  msgHelpBegin       = 5;
  msgHelpEnd         = 20;
  msgOpening         = 21;
  msgReadFromStdin   = 22;
  msgTranscribing    = 23;
  msgDone            = 24;
  msgErr             = 25;
  errParamDuplicated = 26;
  errEmptyOptArg     = 27;
  errParamMandatory  = 28;
  errFileNotOpened   = 29;
  errModlistNotRun   = 30;
  errOtte            = 31-1;

  BadUsage = [
    errParamDuplicated,
    errEmptyOptArg,
    errParamMandatory
  ];

type
  TProgramMessages = array[0..37] of ansistring;

const
  ProgramMessages: TProgramMessages = (
  //policy info - four lines:
    'This program is distributed in the hope that it will be useful,',
    'but WITHOUT ANY WARRANTY; without even the implied warranty of',
    'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the',
    'GNU General Public License for more details.',
  //usage and help messages:
    'usage:',
    '     --version        print version info and exit',
    ' -h, --help           print this help message and exit',
    ' -u, --usage          print short usage briefing and exit',
    ' -i, --input=file     specify input file',
    ' -o, --output=file    specify output file',
    '     --list-modes     list all available modes and exit',
    ' -m, --mode=mode      specify mode file',
    ' -e, --encoding=xxx   specify encoding different than system',
    ' -q, --quiet          be quiet, show only errors',
    ' -B, --back           try to transcribe from tengwar to roman',
    ' -t, --text-nums      treat numbers and spaces like text; suppresses below opts',
    ' -r, --revert-nums    reverse digit order',
    ' -x, --duodecimal     output duodecimal numbers; numbers cut after 18th digit',
    ' -c, --circle-nums    mark duodecades with circles below',
    ' -d, --dot-nums       mark digits with dots above (or below with -x)',
    ' -b, --bar-nums       mark digits with bars above (or below with -x)',
  //verbose messages:
    'loading',
    'reading from stdin...',
    'transcribing',
    'done',
  //word for "error":
    'error: ',
  //usage error messages:
    'parameter duplicated',
    'empty option argument',
    'parameter is mandatory',
  //teng error messages:
    'unable to open file!',
    'unable to run modlist!',
  //OTTE error messages:
    'unable to init character encoding!',
    'unable to open mode file!',
    'unable to read mode file!',
    'mode file size exceeds reasonable limit!',
    'bad format of mode file!',
    'not enough memory!',
    'no mode has been loaded!' //impossible for Teng, but OTTE has such code
  );

{$IFDEF DELPHI}
  var stderr: text;
{$ENDIF}

procedure PrintUsage;
procedure PrintHelp;
procedure PrintVersion;
procedure Die(msg: integer; additional_info: string);
procedure LoadMessages(const fname: AnsiString);

implementation

//-----------------------------------------------------------------------------  
procedure PrintUsage;
begin
  writeln(stderr, ProgramMessages[msgUsage], ' ', ProgramUsage);
end;

//-----------------------------------------------------------------------------  
procedure PrintHelp;

var
  i: integer;
  
begin
  writeln(stderr, ProgramVersion);
  PrintUsage;
  writeln(stderr);
  for i := msgHelpBegin to msgHelpEnd do writeln(stderr, ProgramMessages[i]);
  writeln(stderr);
end;

//-----------------------------------------------------------------------------
procedure PrintVersion;

var
  i: integer;

begin
  writeln(ProgramVersion);
  writeln(ProgramAuthor);
  writeln;
  writeln(ProgramCopyright);
  for i := msgPolicyBegin to msgPolicyEnd do writeln(ProgramMessages[i]);
end;

//-----------------------------------------------------------------------------
procedure Die(msg: integer; additional_info: string);
begin
  writeln(stderr, ProgramMessages[msgErr], additional_info, ': ', ProgramMessages[msg]);
  if msg in BadUsage then PrintHelp;
  halt(1);
end;

//-----------------------------------------------------------------------------  
procedure LoadMessages(const fname: AnsiString);

var
  f: text;
  i: integer;
  NewMessages: TProgramMessages;

begin
  Assign(f, fname);
  Reset(f);
  if IOResult <> 0 then exit; {speak English}

  for i := 0 to high(NewMessages) do
  begin
    if (IOResult <> 0) or eof(f) then begin
      close(f);
      exit; {still speak English}
    end;
    
    Readln(f, NewMessages[i]);
  end;

  ProgramMessages := NewMessages;
  close(f);
end;

//-----------------------------------------------------------------------------  
{$IFDEF DELPHI}
initialization
  assign(stderr, 'CON');
  rewrite(stderr);
finalization
  close(stderr);
{$ENDIF}
end.
